#include <QApplication>
#include "onedeviceelectroprogontask.h"

OneDeviceElectroprogonTask::OneDeviceElectroprogonTask(int currentRevId, int uid, const ChannelFinder::DeviceInfo &devInfo) : QObject(nullptr),
    mCurrentRevId(currentRevId),
    mIsStopped(true),
    mCurrentUid(uid),
    mDevInfo(devInfo)
{
    moveToThread(&mThread);
    mThread.start();
}

OneDeviceElectroprogonTask::~OneDeviceElectroprogonTask()
{
    mThread.quit();
    mThread.wait();
}

void OneDeviceElectroprogonTask::onStart()
{
    auto devStatParams = IWidgetDevice::generateDeviceStatisticsParametresMap();
    mCurrentDevStat = IWidgetDevice::generateDevStat(devStatParams.value(mCurrentRevId));
    mIsStopped = false;
    mTimer = new QTimer;
    mTimerElectroprogon = new QTimer;
    mTimerElectroprogonRx = new QTimer;
    mTimerA708 = new QTimer;
    mTimerOutRk = new QTimer;
    mTimerInRk = new QTimer;
    connect(mTimer, &QTimer::timeout, this, &OneDeviceElectroprogonTask::onTimer);
    connect(mTimerElectroprogon, &QTimer::timeout, this, &OneDeviceElectroprogonTask::onTimerElectroprogon);
    connect(mTimerElectroprogonRx, &QTimer::timeout, this, &OneDeviceElectroprogonTask::onTimerElectroprogonRx);
    connect(mTimerA708, &QTimer::timeout, this, &OneDeviceElectroprogonTask::onTimerA708);
    connect(mTimerOutRk, &QTimer::timeout, this, &OneDeviceElectroprogonTask::onTimerOutRk);
    connect(mTimerInRk, &QTimer::timeout, this, &OneDeviceElectroprogonTask::onTimerInRk);
    initDevice();
    mTimer->start(cTimerPeriod);
    mTimerElectroprogon->start(cTimerElectroprogonPeriod);
    mTimerElectroprogonRx->start(cTimerElectroprogonPeriodRx);
    mTimerA708->start(cTimerA708);
    mTimerOutRk->start(cTimerOutRkPeriod);
    mTimerInRk->start(cTimerInRkPeriod);
}

void OneDeviceElectroprogonTask::onStop()
{  
    mTimer->stop();
    mTimerElectroprogon->stop();
    mTimerA708->stop();
    mTimerOutRk->stop();
    mTimerInRk->stop();
    mTimerElectroprogonRx->stop();
    disconnect(mTimer, &QTimer::timeout, this, &OneDeviceElectroprogonTask::onTimer);
    disconnect(mTimerA708, &QTimer::timeout, this, &OneDeviceElectroprogonTask::onTimerA708);
    disconnect(mTimerElectroprogon, &QTimer::timeout, this, &OneDeviceElectroprogonTask::onTimerElectroprogon);
    disconnect(mTimerElectroprogonRx, &QTimer::timeout, this, &OneDeviceElectroprogonTask::onTimerElectroprogonRx);
    disconnect(mTimerOutRk, &QTimer::timeout, this, &OneDeviceElectroprogonTask::onTimerOutRk);
    disconnect(mTimerInRk, &QTimer::timeout, this, &OneDeviceElectroprogonTask::onTimerInRk);
    delete mTimer;
    delete mTimerElectroprogon;
    delete mTimerElectroprogonRx;
    delete mTimerA708;
    delete mTimerOutRk;
    delete mTimerInRk;
    QEventLoop loop;
    QTimer waitTimer;
    // ожидание завершения корутин-таймеров
    connect(&waitTimer, &QTimer::timeout, this, [this, &loop](void)->void {
        processingTransmitters();
        processingReceivers();
        processingA708();
        if (!loop.isRunning())
            return;

        if ((mIsEndTimer) && (mIsEndTimerElectroprogon) && (mIsEndTimerA708) && (mIsEndTimerElectroprogonRx)) {
            loop.exit();
        }
    });

    // запуск ожидания завершения корутин-таймеров
    waitTimer.start(cTimerElectroprogonPeriod);
    loop.exec();
    // вычёрпывание и печать остатков
    QTimer endTimer;
    QEventLoop endLoop;
    connect(&endTimer, SIGNAL(timeout()), &endLoop, SLOT(quit()));
    endTimer.start(2000);
    endLoop.exec();
    onTimerInRk();
    deinitDevice();
    emit signalSendStatistics(mCurrentUid, mCurrentDevStat);
    mIsStopped = true;
    QTimer tmrDelay;
    QEventLoop loopDelay;
    connect(&tmrDelay, &QTimer::timeout, this, [&tmrDelay, &loopDelay, this](void)->void {
        processingTransmitters();
        processingReceivers();
        processingA708();
        if (loopDelay.isRunning() == false) return;
        tmrDelay.stop();
        loopDelay.exit();
    });
    tmrDelay.start(1000);
    loopDelay.exec();
    emit signalSendStatistics(mCurrentUid, mCurrentDevStat);
    emit signalHasFinished();
}

void OneDeviceElectroprogonTask::onElectroprogonParametresSet(ElectroprogonParametres params)
{
    mParams = params;
}

void OneDeviceElectroprogonTask::onTimer()
{
    if (mIsStopped)
        return;
    mIsEndTimer = false;
    // отправка наработанной статистики
    emit signalSendStatistics(mCurrentUid, mCurrentDevStat);
    mIsEndTimer = true;
}

void OneDeviceElectroprogonTask::onTimerElectroprogon()
{
    if (mIsStopped)
        return;
    mIsEndTimerElectroprogon = false;
    // обработка передатчиков
    processingTransmitters();
    mIsEndTimerElectroprogon = true;
}

void OneDeviceElectroprogonTask::onTimerElectroprogonRx()
{
    if (mIsStopped)
        return;
    mIsEndTimerElectroprogonRx = false;
    // обработка приёмников
    processingReceivers();
    mIsEndTimerElectroprogonRx = true;
}

void OneDeviceElectroprogonTask::onTimerA708()
{
    if (mIsStopped)
        return;
    mIsEndTimerA708 = false;
    processingA708();
    mIsEndTimerA708 = true;
}

void OneDeviceElectroprogonTask::onTimerOutRk()
{
    if (mIsStopped)
        return;
    int fd = a708_open(mDevInfo.mDevName.toLatin1().constData());
    // выдача рк
    a708_manageScOut(fd, A708_SC_OUT_1, A708_SC_OUT_1, A708_SC_OUT_1, A708_SC_OUT_1);
    if (mCurrentDevStat.mRkOut.count() >= 1)
        mCurrentDevStat.mRkOut[0]++;
    if (mCurrentDevStat.mRkOut.count() >= 2)
        mCurrentDevStat.mRkOut[1]++;
    if (mCurrentDevStat.mRkOut.count() >= 3)
        mCurrentDevStat.mRkOut[2]++;
    if (mCurrentDevStat.mRkOut.count() >= 4)
        mCurrentDevStat.mRkOut[3]++;
    a708_manageScOut(fd, A708_SC_OUT_0, A708_SC_OUT_0, A708_SC_OUT_0, A708_SC_OUT_0);
    a708_close(fd);
}

void OneDeviceElectroprogonTask::onTimerInRk()
{
    if (mIsStopped)
        return;
    int fd = a708_open(mDevInfo.mDevName.toLatin1().constData());
    // приём рк
    T_CHECK_TRACKING_INT trackCheckInt;
    trackCheckInt.finishTrackingFreeTimer = 0;
    trackCheckInt.startTrackingFreeTimer = 0;
    trackCheckInt.rkInAppearCount[0] = trackCheckInt.rkInAppearCount[1] = trackCheckInt.rkInAppearCount[2] = trackCheckInt.rkInAppearCount[3] = 0;
    // проверяем результат ВХ РК
    a708_checkTrackingInterrupt(fd, &trackCheckInt);
    if (mCurrentDevStat.mRkIn.count() >= 1)
        mCurrentDevStat.mRkIn[T_CHECK_TRACKING_INT_RKIN_1] = trackCheckInt.rkInAppearCount[T_CHECK_TRACKING_INT_RKIN_1];
    if (mCurrentDevStat.mRkIn.count() >= 2)
        mCurrentDevStat.mRkIn[T_CHECK_TRACKING_INT_RKIN_2] = trackCheckInt.rkInAppearCount[T_CHECK_TRACKING_INT_RKIN_2];
    if (mCurrentDevStat.mRkIn.count() >= 3)
        mCurrentDevStat.mRkIn[T_CHECK_TRACKING_INT_RKIN_3] = trackCheckInt.rkInAppearCount[T_CHECK_TRACKING_INT_RKIN_3];
    if (mCurrentDevStat.mRkIn.count() >= 4)
        mCurrentDevStat.mRkIn[T_CHECK_TRACKING_INT_RKIN_4] = trackCheckInt.rkInAppearCount[T_CHECK_TRACKING_INT_RKIN_4];
    a708_close(fd);
}

void OneDeviceElectroprogonTask::initDevice()
{
    int fdev = a708_open(mDevInfo.mDevName.toLatin1().constData());
//    if (fdev == -1) {
//        emit signalPrint(QString("Can't open dev = '%1'").arg(mDevInfo.mDevName));
//        return;
//    }
    a708_manageDma(fdev, A708_ON);
    a708_clearDma(fdev);
    a708_close(fdev);
    // инициализация каналов
    auto receivers = ChannelFinder::getReceiveChannelNameListByNumDevice(mDevInfo.mDevNum);
    foreach (QString receiver, receivers) {
        int fd = a708_open(receiver.toLatin1().constData());
        a708_ramCls(fd);
        QTimer tmrDelay;
        QEventLoop loopDelay;
        connect(&tmrDelay, &QTimer::timeout, this, [&tmrDelay, &loopDelay](void)->void {
            if (loopDelay.isRunning() == false) return;
             tmrDelay.stop();
             loopDelay.exit();
        });
        tmrDelay.start(10);
        loopDelay.exec();
        a708_deinit(fd);
        a708_manageChannelDma(fd, A708_ON);
        a708_setSpeed(fd, mParams.mSpeed);
        a708_parityManage(fd, A708_ON, A708_ON);
        a708_manageReverce(fd, A708_OFF);
        a708_manageEnBit(fd, A708_ON);
        a708_close(fd);
    }
    auto transmitters = ChannelFinder::getTransmitChannelNameListByNumDevice(mDevInfo.mDevNum);
    foreach (QString transmitter, transmitters) {
        int fd = a708_open(transmitter.toLatin1().constData());
        a708_ramCls(fd);
        QTimer tmrDelay;
        QEventLoop loopDelay;
        connect(&tmrDelay, &QTimer::timeout, this, [&tmrDelay, &loopDelay](void)->void {
            if (loopDelay.isRunning() == false) return;
             tmrDelay.stop();
             loopDelay.exit();
        });
        tmrDelay.start(10);
        loopDelay.exec();
        a708_deinit(fd);
        a708_manageEnBit(fd, A708_OFF);
        a708_txGapBits(fd, 4);
        a708_manageChannelDma(fd, A708_ON);
        a708_setSpeed(fd, mParams.mSpeed);
        a708_parityManage(fd, A708_ON, A708_ON);
        a708_manageReverce(fd, A708_OFF);
        a708_txSetMode(fd, A708_MODE_1);
        a708_txRrD(fd, mParams.mDiscrRRT);
        a708_manageEnBit(fd, A708_ON);
        a708_close(fd);
    }
    // инициализация а708
    auto a708chs = ChannelFinder::getA708ChannelNameListByNumDevice(mDevInfo.mDevNum);
    int a708num = 0;
    foreach (QString a708ch, a708chs) {
        a708num++;
        int fd = a708_open(a708ch.toLatin1().constData());
        if (a708num == mParams.mNumA708T) {
            a708_tx708_setMode(fd, A708_TX708_MODE_SINGLE_SHOT);
            a708_rx708_manage(fd, A708_ON);
            a708_rx708_shortDma(fd, A708_ON);
        } else {
            a708_rx708_manage(fd, A708_ON);
            a708_rx708_shortDma(fd, A708_OFF);
        }
        a708_close(fd);
    }
    for (int i=0; i<A708_DATA_BLOCK_SIZE_708; i++){
        mBlockA708.words[i] = 0xAAAA;
    }
    mBlockA708.length = A708_DATA_BLOCK_SIZE_708;
    mTrigger = true;
    writeMem();
    // инициализация вых рк
    int fd = a708_open(mDevInfo.mDevName.toLatin1().constData());
    a708_manageInterruptMask(fd, 1<<8 | 1<<9 | 1<<10 | 1<<11, 1<<8 | 1<<9 | 1<<10 | 1<<11);
    a708_manageScIntMask(fd, A708_IN_RK_1, A708_SC_FALL, A708_ON);
    a708_manageScIntMask(fd, A708_IN_RK_2, A708_SC_FALL, A708_ON);
    a708_manageScIntMask(fd, A708_IN_RK_3, A708_SC_FALL, A708_ON);
    a708_manageScIntMask(fd, A708_IN_RK_4, A708_SC_FALL, A708_ON);
    T_TRACKING_INT trackInt;
    // включаем слежение за РК ВХ
    trackInt.manage = T_TRACKING_INT_MAN_ON;
    a708_manageTrackingInterrupt(fd, &trackInt);
    a708_close(fd);
    // старт каналов
    foreach (QString transmitter, transmitters) {
        int fd = a708_open(transmitter.toLatin1().constData());
        if (mParams.mOnRRT) {
            a708_txStartWithRrt(fd, mParams.mValueRRT);
        } else {
            a708_txStartStop(fd, A708_START);
        }
        a708_close(fd);
    }
}

void OneDeviceElectroprogonTask::deinitDevice()
{
    // останов каналов
    auto transmitters = ChannelFinder::getTransmitChannelNameListByNumDevice(mDevInfo.mDevNum);
    foreach (QString transmitter, transmitters) {
        int fd = a708_open(transmitter.toLatin1().constData());
        a708_txStartStop(fd, A708_STOP);
        a708_close(fd);
    }
    QTimer tmrDelay;
    QEventLoop loopDelay;
    connect(&tmrDelay, &QTimer::timeout, this, [&tmrDelay, &loopDelay](void)->void {
        if (loopDelay.isRunning() == false) return;
         tmrDelay.stop();
         loopDelay.exit();
    });
    tmrDelay.start(500);
    loopDelay.exec();
    // ДЕинициализация вых рк
    int fd = a708_open(mDevInfo.mDevName.toLatin1().constData());
    T_TRACKING_INT trackInt;
    // включаем слежение за РК ВХ
    trackInt.manage = T_TRACKING_INT_MAN_OFF;
    a708_manageTrackingInterrupt(fd, &trackInt);
    a708_manageDma(fd, A708_OFF);
    a708_close(fd);
    auto receivers = ChannelFinder::getReceiveChannelNameListByNumDevice(mDevInfo.mDevNum);
    // ДЕинициализация каналов
    foreach (QString transmitter, transmitters) {
        int fd = a708_open(transmitter.toLatin1().constData());
        a708_manageEnBit(fd, A708_OFF);
        a708_ramCls(fd);
        a708_deinit(fd);
        a708_close(fd);
    }
    foreach (QString receiver, receivers) {
        int fd = a708_open(receiver.toLatin1().constData());
        a708_manageEnBit(fd, A708_OFF);
        a708_ramCls(fd);
        a708_deinit(fd);
        a708_close(fd);
    }
    // ДЕинициализация а708
    auto a708chs = ChannelFinder::getA708ChannelNameListByNumDevice(mDevInfo.mDevNum);
    foreach (QString a708ch, a708chs) {
        int fd = a708_open(a708ch.toLatin1().constData());
        a708_rx708_manage(fd, A708_OFF);
        a708_tx708_setMode(fd, A708_TX708_MODE_OFF);
        a708_rx708_shortDma(fd, A708_OFF);
        a708_close(fd);
    }
}

void OneDeviceElectroprogonTask::processingTransmitters()
{
    auto transmitters = ChannelFinder::getTransmitChannelNameListByNumDevice(mDevInfo.mDevNum);
    foreach (QString transmitter, transmitters) {
        // запросы к каналу
        int fd = a708_open(transmitter.toLatin1().constData());
        if (fd == -1) {
            emit signalPrint(QString("Can't open dev = '%1'").arg(transmitter));
            continue;
        }
        T_INFO_CHANNEL info;
        a708_getChannelInfo(fd, &info);
        T_CONTAINER_DMA dma;
        dma.count = 0;
        a708_readDma(fd, &dma);
        a708_close(fd);
        // учёт отправленных сообщений
        mCurrentDevStat.mTMsg[info.numberChannel] += dma.count;
    }
}

void OneDeviceElectroprogonTask::processingReceivers()
{
    auto printErrA429Rcv = [this](T_INFO_CHANNEL* ch, T_CONTAINER_DMA* dma)->void {
        auto hasDmaBlockA429RcvErr = [this](T_BLOCK_DMA* block)->int {
            if (block->word2 & (1<<21)) {
                return 1;
            }
            if (block->word2 & (1<<22)) {
                return 1;
            }
            if (block->word2 & (1<<23)) {
                return 1;
            }
            return 0;
        };
        if (ch->typeChannel == T_INFO_CHANNEL_TYPE_RECIEVER) {
            for (int i=0; i < dma->count; i++) {
                if (hasDmaBlockA429RcvErr( &dma->blocks[i] )) {
                    emit signalPrint(QString("Error in A429 rcv ch %1 dma word2= 0x%2:")
                                     .arg(QString::number(ch->numberChannel))
                                     .arg(QString::number(dma->blocks[i].word2, 16))
                                     );
                    if (dma->blocks[i].word2 & (1<<21)) {
                        emit signalPrint("Ошибка кодировки бита");
                    }
                    if (dma->blocks[i].word2 & (1<<22)) {
                        emit signalPrint("Ошибка паузы");
                    }
                    if (dma->blocks[i].word2 & (1<<23)) {
                        emit signalPrint("Ошибка чётности");
                    }
                }
            }
        }
    };
    auto receivers = ChannelFinder::getReceiveChannelNameListByNumDevice(mDevInfo.mDevNum);
    foreach (QString receiver, receivers) {
        // запросы к каналу
        int fd = a708_open(receiver.toLatin1().constData());
        if (fd == -1) {
            emit signalPrint(QString("Can't open dev = '%1'").arg(receiver));
            continue;
        }
        T_INFO_CHANNEL info;
        a708_getChannelInfo(fd, &info);
        T_CONTAINER_DMA dma;
        dma.count = 0;
        a708_readDma(fd, &dma);
        a708_close(fd);
        // учёт отправленных сообщений
        mCurrentDevStat.mRMsg[info.numberChannel] += dma.count;
        // проверка на наличие ошибок и их учёт
        for (auto index=0; index<static_cast<int>(dma.count); index++) {
            if (dma.blocks[index].word2 & ( 1<<21 | 1<<22 | 1<<23)) {
               mCurrentDevStat.mRErr[info.numberChannel] += 1;
            }
        }
        printErrA429Rcv(&info, &dma);
    }
}

void OneDeviceElectroprogonTask::processingA708()
{
    auto printErrA708 = [this](T_INFO_CHANNEL* ch, T_CONTAINER_DMA_A708* dma)->void {
        auto hasDmaBlockA708Err = [this](T_BLOCK_DMA_A708* block)->int {
            if (block->words[0] & (1<<16)) {
                return 1;
            }
            if (block->words[0] & (1<<17)) {
                return 1;
            }
            if (block->words[0] & (1<<18)) {
                return 1;
            }
            if (block->words[0] & (1<<19)) {
                return 1;
            }
            return 0;
        };
        if (ch->typeChannel == T_INFO_CHANNEL_TYPE_A708) {
            for (int i=0; i < dma->count; i++) {
                if (hasDmaBlockA708Err( &dma->blocks[i] )) {
                    emit signalPrint(QString("Error in A708 rcv ch %1 dma word1= 0x%2:")
                                     .arg(QString::number(ch->numberChannel))
                                     .arg(QString::number(dma->blocks[i].words[0], 16))
                                     );
                    if (dma->blocks[i].words[0] & (1<<16)) {
                        emit signalPrint("Oшибка кодировки «манчестер»");
                    }
                    if (dma->blocks[i].words[0] & (1<<17)) {
                        emit signalPrint("Oшибка длины");
                    }
                    if (dma->blocks[i].words[0] & (1<<18)) {
                        emit signalPrint("Oшибка синхронизации 1");
                    }
                    if (dma->blocks[i].words[0] & (1<<19)) {
                        emit signalPrint("Oшибка синхронизации 2");
                    }
                }
            }
        }
    };
    auto isErrInDmaBlock = [](quint32 word) -> bool {
        bool isErr = false;
        if ((word & 1<<16) > 0) isErr = true;
        if ((word & 1<<17) > 0) isErr = true;
        if ((word & 1<<18) > 0) isErr = true;
        if ((word & 1<<19) > 0) isErr = true;
        return isErr;
    };
    auto a708chs = ChannelFinder::getA708ChannelNameListByNumDevice(mDevInfo.mDevNum);
    foreach (QString a708ch, a708chs) {
        // запросы к каналу
        int fd = a708_open(a708ch.toLatin1().constData());
        if (fd == -1) {
            emit signalPrint(QString("Can't open dev = '%1'").arg(a708ch));
            continue;
        }
        T_INFO_CHANNEL info;
        a708_getChannelInfo(fd, &info);
        T_CONTAINER_DMA_A708 dma;
        if (mParams.mNumA708T-1 == info.numberChannel) {
            a708_readDmaA708(fd, &dma);
            mCurrentDevStat.mA708T[info.numberChannel] += dma.count;
            if (dma.count > DMA_COUNT_BLOCKS_A708) {
                emit signalPrint(QString("dma.count{%2} > DMA_COUNT_BLOCKS_A708 - dev = '%1'")
                                 .arg(a708ch)
                                 .arg(QString::number(dma.count)));
                goto CLOSE;
            }
            for (int i=0; i<dma.count; i++) {
                if (isErrInDmaBlock(dma.blocks[i].words[1-1]))
                    mCurrentDevStat.mA708Err[info.numberChannel] += 1;
            }
            if (mTrigger)
                a708_tx708_writeDataBlock(fd, 1, &mBlockA708);
            else
                a708_tx708_start(fd, A708_TX708_MODE_SINGLE_SHOT);
            mTrigger = !mTrigger;
        } else {
            a708_readDmaA708(fd, &dma);
            mCurrentDevStat.mA708R[info.numberChannel] += dma.count;
            for (int i=0; i<dma.count; i++) {
                if (isErrInDmaBlock(dma.blocks[i].words[1-1]))
                    mCurrentDevStat.mA708Err[info.numberChannel] += 1;
            }
            printErrA708(&info, &dma);
        }
        CLOSE:
        a708_close(fd);
    }
}

bool OneDeviceElectroprogonTask::writeMem()
{
    bool isError = false;
    QVector<MemElement> mem;
    QString memName = QString("%1/mem/%2").arg(QApplication::applicationDirPath()).arg("desk_test_progon.mem");
    auto isReaded = MemLoader::loadMemFile(memName, mem);
    if (!isReaded) {
        isError = true;
        emit signalPrint(QString("dev: '%1' - не удалось считать mem файл: '%2'")
                         .arg(mDevInfo.mDevName)
                         .arg(memName)
                         );
    }
    auto transmitters = ChannelFinder::getTransmitChannelNameListByNumDevice(mDevInfo.mDevNum);
    QVector<MemElement> data;
    for (int i=0; i<256; i++){
        data.append(MemElement(i, 0xAAAAAAAA));
    }
    foreach (QString transmitter, transmitters) {
        bool isWrited;
        isWrited = MemLoader::writeToChannel(transmitter, mem);
        if (!isWrited) {
            isError = true;
            emit signalPrint(QString("dev: '%1' - не удалось загрузить mem файл в передатчик '%2'")
                             .arg(mDevInfo.mDevName)
                             .arg(transmitter)
                             );
        }
        isWrited = MemLoader::writeDataToChannel(transmitter, data);
        if (!isWrited) {
            isError = true;
            emit signalPrint(QString("dev: '%1' - не удалось загрузить данные в передатчик '%2'")
                             .arg(mDevInfo.mDevName)
                             .arg(transmitter)
                             );
        }
    }
    return !isError;
}
