#include <QDir>
#include <QRegExp>

#include "channelfinder.h"
#include "a708_library.h"

bool ChannelFinder::sortCondition(const QString &a, const QString &b)
{
    auto listA = a.split('-');
    auto listB = b.split('-');

    if (static_cast<QString>(listA.at(1)).toInt() == static_cast<QString>(listB.at(1)).toInt())
        return static_cast<QString>(listA.at(3)).toInt() < static_cast<QString>(listB.at(3)).toInt();
    else
       return static_cast<QString>(listA.at(1)).toInt() < static_cast<QString>(listB.at(1)).toInt() ;
}

QStringList ChannelFinder::getChannelNameList()
{
#if 1 // РЕАЛЬНЫЕ КАНАЛЫ
    QDir dev("/dev");
    QStringList allFiles = dev.entryList(QDir::System);
    QRegExp rx("a708dev-(\\d+)-ch-(\\d+)");
    // обыскать каталог dev и найти паттерны
    QStringList channels;
    foreach (QString fileName, allFiles) {
        if (rx.exactMatch(fileName)) {
            channels.append(QString("/dev/%1").arg(fileName));
        }
    }
    qSort(channels.begin(), channels.end(), sortCondition);
    return channels;
#else // ПОДЫГРЫШ для ОТЛАДКИ
    QStringList channels;
    for(int i=0; i < 4; i++)
        for(int j=0; j < 4; j++)
            channels.append(QString("mil1553dev-%1-ch-%2")
                            .arg(QString::number(i))
                            .arg(QString::number(j))
                            );
    return channels;
#endif
}

QStringList ChannelFinder::getDeviceNameList()
{
    QStringList channels = getChannelNameList();
    QRegExp rx("/dev/a708dev-(\\d+)-ch-0");
    QStringList devices;
    foreach (QString fileName, channels) {
        if (rx.exactMatch(fileName)) {
            devices.append(fileName);
        }
    }
    return devices;
}

QVector<ChannelFinder::ChannelInfo> ChannelFinder::getChannelInfoList()
{
#if 1 // РЕАЛЬНЫЕ КАНАЛЫ
    QVector<ChannelFinder::ChannelInfo> infoChannels;
    QStringList channels = getChannelNameList();
    foreach (QString fileName, channels) { // запросить у каналов их pciLocation
        int dev = open(fileName.toLatin1().constData(), O_RDWR);
        PCI_LOCATION pciLoc; // pci локация
        ioctl(dev, IOCTL_PCI_LOCATION, &pciLoc);
        ChannelFinder::ChannelInfo info;
        info.m_name = fileName;
        info.m_pciLocation = QString::fromLatin1(pciLoc.name, SIZE_OF_PCI_LOC);
        infoChannels.append(info);
        close(dev);
    }
    return infoChannels;
#else // ПОДЫГРЫШ для ОТЛАДКИ
    QVector<ChannelFinder::ChannelInfo> infoChannels;
    QStringList channels = getChannelList();
    foreach (QString fileName, channels) {
        ChannelFinder::ChannelInfo info;
        info.m_name = fileName;
        info.m_pciLocation = QString("00:00:00:0");
        infoChannels.append(info);
    }
    return infoChannels;
#endif
}

QVector<ChannelFinder::DeviceInfo> ChannelFinder::getDeviceInfoList()
{
    auto devNames = ChannelFinder::getDeviceNameList();
    QVector<ChannelFinder::DeviceInfo> devices;
    for(int i=0; i<devNames.count(); i++) {
        ChannelFinder::DeviceInfo dev;
        dev.mDevName = devNames[i];
        dev.mDevNum = i;
        devices.append(dev);
    }
    return devices;
}

int ChannelFinder::getCountDevices()
{
    QStringList channels = getChannelNameList();
    QRegExp rx("/dev/a708dev-(\\d+)-ch-0");
    int count = 0;
    foreach (QString fileName, channels) {
        if (rx.exactMatch(fileName)) {
            count++;
        }
    }
    return count;
}

QStringList ChannelFinder::getChannelNameListByNumDevice(int numDev)
{
    QDir dev("/dev");
    QStringList allFiles = dev.entryList(QDir::System);
    QRegExp rx(QString("a708dev-%1-ch-(\\d+)")
               .arg(QString::number(numDev))
               );
    // обыскать каталог dev и найти паттерны
    QStringList channels;
    foreach (QString fileName, allFiles) {
        if (rx.exactMatch(fileName)) {
            channels.append(QString("/dev/%1").arg(fileName));
        }
    }
    qSort(channels.begin(), channels.end(), sortCondition);
    return channels;
}

QStringList ChannelFinder::getReceiveChannelNameListByNumDevice(int numDev)
{
    auto channels = ChannelFinder::getChannelNameListByNumDevice(numDev);
    QStringList receivers;
    foreach (QString fileName, channels) {
        int fd = a708_open(fileName.toLatin1().constData());
        T_INFO_CHANNEL info;
        a708_getChannelInfo(fd, &info);
        if (info.typeChannel == T_INFO_CHANNEL_TYPE_RECIEVER)
            receivers.append(fileName);
        a708_close(fd);
    }
    qSort(channels.begin(), channels.end(), sortCondition);
    return receivers;
}

QStringList ChannelFinder::getTransmitChannelNameListByNumDevice(int numDev)
{
    auto channels = ChannelFinder::getChannelNameListByNumDevice(numDev);
    QStringList transmitters;
    foreach (QString fileName, channels) {
        int fd = a708_open(fileName.toLatin1().constData());
        T_INFO_CHANNEL info;
        a708_getChannelInfo(fd, &info);
        if (info.typeChannel == T_INFO_CHANNEL_TYPE_TRANSMITTER)
            transmitters.append(fileName);
        a708_close(fd);
    }
    qSort(channels.begin(), channels.end(), sortCondition);
    return transmitters;
}

QStringList ChannelFinder::getA708ChannelNameListByNumDevice(int numDev)
{
    auto channels = ChannelFinder::getChannelNameListByNumDevice(numDev);
    QStringList a708chs;
    foreach (QString fileName, channels) {
        int fd = a708_open(fileName.toLatin1().constData());
        T_INFO_CHANNEL info;
        a708_getChannelInfo(fd, &info);
        if (info.typeChannel == T_INFO_CHANNEL_TYPE_A708)
            a708chs.append(fileName);
        a708_close(fd);
    }
    qSort(channels.begin(), channels.end(), sortCondition);
    return a708chs;
}

QString ChannelFinder::getChannelByNumberSince0(QStringList channels, int number)
{
    foreach (QString channel, channels) {
        int fd = a708_open(channel.toLatin1().constData());
        T_INFO_CHANNEL info;
        a708_getChannelInfo(fd, &info);
        a708_close(fd);
        if (info.numberChannel == number)
            return channel;
    }
    return "";
}

QString ChannelFinder::ChannelInfo::toString()
{
    // перевод role в строковое значение
    return QString("%1 (%2)").arg(m_name).arg(m_pciLocation);
}

