#ifndef CHANNEL_FINDER_H
#define CHANNEL_FINDER_H

#include <QObject>
#include <QStringList>
#include <QVector>

///
/// \brief The ChannelFinder class поисковик каналов
///
class ChannelFinder
{
private:
    ///
    /// \brief sortCondition условие сортировки каналов плат
    /// \param a текущий
    /// \param b следующий
    /// \return результат
    ///
    static bool sortCondition(const QString& a, const QString& b);
public:
    ///
    /// \brief The ChannelInfo struct информация о канале
    ///
    struct ChannelInfo {
        QString m_name;             ///< полное имя канала
        QString m_pciLocation;      ///< информация об адресе pci
        /// \brief toString строковое имя
        /// \return строка
        QString toString();
    };
    ///
    /// \brief The DeviceInfo struct информация о плате
    ///
    struct DeviceInfo {
        QString mDevName;           ///< имя платы
        int mDevNum;                ///< номер платы
    };
    /// \brief getChannelNameList получить список имён каналов
    /// \return список имён каналов
    static QStringList getChannelNameList();
    /// \brief getDeviceNameList получить список плат
    /// \return список плат
    static QStringList getDeviceNameList();
    /// \brief getChannelInfoList получить список с информацией о каналах
    /// \return список с информацией о каналах
    static QVector<ChannelInfo> getChannelInfoList();
    /// \brief getDeviceInfoList получить список плат
    /// \return список плат
    static QVector<DeviceInfo> getDeviceInfoList();
    ///
    /// \brief getCountDevices получить кол-во девайсов
    /// \return кол-во девайсов
    ///
    static int getCountDevices();
    ///
    /// \brief getChannelNameListByNumDevice получить список каналов по номеру платы
    /// \param numDev номер платы
    /// \return список каналов
    ///
    static QStringList getChannelNameListByNumDevice(int numDev);
    ///
    /// \brief getReceiveChannelNameListByNumDevice получить список приёмников платы
    /// \param numDev номер девайса
    /// \return список каналов
    ///
    static QStringList getReceiveChannelNameListByNumDevice(int numDev);
    ///
    /// \brief getTransmitChannelNameListByNumDevice получить список передатчиков платы
    /// \param numDev номер девайса
    /// \return список каналов
    ///
    static QStringList getTransmitChannelNameListByNumDevice(int numDev);
    ///
    /// \brief getA708ChannelNameListByNumDevice получить список a708 платы
    /// \param numDev номер девайса
    /// \return список каналов
    ///
    static QStringList getA708ChannelNameListByNumDevice(int numDev);
    ///
    /// \brief getChannelByNumberSince0 получить имя канала по его номеру с 0
    /// \param channels каналы
    /// \param number номер
    /// \return имя канала
    ///
    static QString getChannelByNumberSince0(QStringList channels, int number);
};

#endif // CHANNEL_FINDER_H

