#include "globaltools.h"

quint32 GlobalTools::readRegRxConf(const QString &channelName)
{
    T_INFO_CHANNEL info;
    int fd = a708_open(channelName.toLatin1().constData());
    a708_getChannelInfo(fd, &info);
    if (info.typeChannel == T_INFO_CHANNEL_TYPE_TRANSMITTER)
        return 0xFFFFFFFF;
    SADDR_DATA reg;
    reg.daddr = REG_RX_CONF(info.numberChannel);
    a708_readReg(fd, &reg);
    a708_close(fd);
    return reg.data;
}

quint32 GlobalTools::readRegTxConf(const QString &channelName)
{
    T_INFO_CHANNEL info;
    int fd = a708_open(channelName.toLatin1().constData());
    a708_getChannelInfo(fd, &info);
    if (info.typeChannel == T_INFO_CHANNEL_TYPE_RECIEVER)
        return 0xFFFFFFFF;
    SADDR_DATA reg;
    reg.daddr = REG_TX_CONF(info.numberChannel);
    a708_readReg(fd, &reg);
    a708_close(fd);
    return reg.data;
}

quint32 GlobalTools::readRegTxControl(const QString &channelName)
{
    T_INFO_CHANNEL info;
    int fd = a708_open(channelName.toLatin1().constData());
    a708_getChannelInfo(fd, &info);
    if (info.typeChannel == T_INFO_CHANNEL_TYPE_RECIEVER)
        return 0xFFFFFFFF;
    SADDR_DATA reg;
    reg.daddr = REG_TX_CONTROL(info.numberChannel);
    a708_readReg(fd, &reg);
    a708_close(fd);
    return reg.data;
}

QString GlobalTools::convertValToBinStr(quint32 value)
{
    auto line = QString::number(value, 2).rightJustified(32, '0').toUpper();
    auto resultLine = QString("%1 %2 %3 %4")
            .arg(line.mid(0, 8))
            .arg(line.mid(8, 8))
            .arg(line.mid(16, 8))
            .arg(line.mid(24, 8));
    return resultLine;
}

QString GlobalTools::convertValToHexStr(quint32 value)
{
    auto line = QString::number(value, 16).rightJustified(8, '0').toUpper();
    auto resultLine = QString("0x%1")
            .arg(line);
    return resultLine;
}

int GlobalTools::readRegRxLabelConf(const QString &channelName, int label, int &numReg, quint32 &valReg)
{
    unsigned int ret = -1;
    SADDR_DATA reg;
    T_INFO_CHANNEL chInfo;
    int fd = a708_open(channelName.toLatin1().constData());
    numReg = -1;
    valReg = 0;
    ret = a708_getChannelInfo(fd, &chInfo);
    if (chInfo.typeChannel == T_INFO_CHANNEL_TYPE_TRANSMITTER) {
        goto end;
    }
    if ((label >= 0) && (label <= 31)) {
        reg.daddr = REG_LBL_CONF_PCI_0(chInfo.numberChannel);
        numReg = 0;
    } else if ((label >= 32) && (label <= 63)) {
        reg.daddr = REG_LBL_CONF_PCI_1(chInfo.numberChannel);
        numReg = 1;
    } else if ((label >= 64) && (label <= 95)) {
        reg.daddr = REG_LBL_CONF_PCI_2(chInfo.numberChannel);
        numReg = 2;
    } else if ((label >= 96) && (label <= 127)) {
        reg.daddr = REG_LBL_CONF_PCI_3(chInfo.numberChannel);
        numReg = 3;
    } else if ((label >= 128) && (label <= 159)) {
        reg.daddr = REG_LBL_CONF_PCI_4(chInfo.numberChannel);
        numReg = 4;
    } else if ((label >= 160) && (label <= 191)) {
        reg.daddr = REG_LBL_CONF_PCI_5(chInfo.numberChannel);
        numReg = 5;
    } else if ((label >= 192) && (label <= 223)) {
        reg.daddr = REG_LBL_CONF_PCI_6(chInfo.numberChannel);
        numReg = 6;
    } else if ((label >= 224) && (label <= 255)) {
        reg.daddr = REG_LBL_CONF_PCI_7(chInfo.numberChannel);
        numReg = 7;
    } else
        goto end;
    ret = a708_readReg(fd, &reg);
    valReg = reg.data;
    end:
    a708_close(fd);
    return ret;
}

quint32 GlobalTools::mirroredLabel(quint32 data)
{
    quint32 mirroredData = 0;
    int shift = 7;
    for (int index = 0; index < 8; index++, shift--) {
        mirroredData |= (data & (1<<index)) ? (1 << shift) : 0;
    }
    mirroredData |= (data & 0xFFFFFF00);
    return mirroredData;
}
