#ifndef GLOBALTOOLS_H
#define GLOBALTOOLS_H

#include <QObject>
#include "a708_library.h"

///
/// \brief The GlobalTools class глобальные инструменты
///
class GlobalTools
{
public:
    ///
    /// \brief readRegRxConf прочитать регистр RX_CONF
    /// \param channelName имя канала
    /// \return значение регистра
    ///
    static quint32 readRegRxConf(const QString& channelName);
    ///
    /// \brief readRegTxConf прочитать регистр TX_CONF
    /// \param channelName имя канала
    /// \return значение регистра
    ///
    static quint32 readRegTxConf(const QString& channelName);
    ///
    /// \brief readRegTxControl прочитать регистр TX_CONTROL
    /// \param channelName имя канала
    /// \return значение регистра
    ///
    static quint32 readRegTxControl(const QString& channelName);
    ///
    /// \brief convertU32toBinString перевод числа в двоичный текстовый вид
    /// \param value число
    /// \return двоичный текстовый вид
    ///
    static QString convertValToBinStr(quint32 value);
    ///
    /// \brief convertValToHexStr перевод числа в шестнадцатеричный текстовый вид
    /// \param value число
    /// \return шестнадцатеричный текстовый вид
    ///
    static QString convertValToHexStr(quint32 value);
    ///
    /// \brief readRegRxLabelConf Чтение регистра фильтра, соотв. метке
    /// \param channelName имя канала
    /// \param label метка
    /// \param numReg номер регистра фильтра
    /// \param valReg значение регистра
    /// \return код результата
    ///
    static int readRegRxLabelConf(const QString& channelName, int label, int& numReg, quint32& valReg);
    ///
    /// \brief mirroredLabel отзеркалить метку (адрес)
    /// \param data слово дпк
    /// \return отзеркаленная метка
    ///
    static quint32 mirroredLabel(quint32 data);
};

#endif // GLOBALTOOLS_H
