#include <QDateTime>
#include <QDir>

#include "logger.h"

Logger::Logger(QObject *parent) : QObject(parent)
{
    mOutputDirectory = QString("%1/logs").arg(QApplication::applicationDirPath());
    mPrefixName = "";
    mOutputDirectoryHtml = QString("%1/logs").arg(QApplication::applicationDirPath());
    mPrefixNameHtml = "";
}

void Logger::setPrefixName(const QString &prefix)
{
    mPrefixName = prefix;
    mPrefixNameHtml = prefix;
}

void Logger::setOutputDirectory(const QString &dir)
{
    mOutputDirectory = dir;
    mOutputDirectoryHtml = dir;
}

void Logger::startLog()
{
    QDir dir(mOutputDirectory);
    if (!dir.exists()) {
        dir.mkpath(mOutputDirectory);
    }
    if (mLogFile.isOpen()) {
        mLogFile.close();
    }
    mPrevFileName = QString("%1/%3_%2.txt")
            .arg(mOutputDirectory)
            .arg(QDateTime::currentDateTime().toString("dd-MM-yyyy_hh-mm-ss"))
            .arg(mPrefixName);
    mLogFile.setFileName(mPrevFileName
                         );
    if (mLogFile.open(QFile::Append)) {
        mStream.setDevice(&mLogFile);
        mIsRunned = true;
    }
    //---
    QDir dirHtml(mOutputDirectoryHtml);
    if (!dirHtml.exists()) {
        dirHtml.mkpath(mOutputDirectoryHtml);
    }
    if (mLogFileHtml.isOpen()) {
        mLogFileHtml.close();
    }
    mPrevFileNameHtml = QString("%1/%3_%2.html")
            .arg(mOutputDirectoryHtml)
            .arg(QDateTime::currentDateTime().toString("dd-MM-yyyy_hh-mm-ss"))
            .arg(mPrefixNameHtml);
    mLogFileHtml.setFileName(mPrevFileNameHtml
                         );
    if (mLogFileHtml.open(QFile::Append)) {
        mStreamHtml.setDevice(&mLogFileHtml);
        mIsRunnedHtml = true;
    }
}

void Logger::resumeLog()
{
    if (!mIsRunned){
        QDir dir(mOutputDirectory);
        if (!dir.exists()) {
            dir.mkpath(mOutputDirectory);
        }
        if (mLogFile.isOpen()) {
            mLogFile.close();
        }
        mLogFile.setFileName(mPrevFileName
                             );
        if (mLogFile.open(QFile::Append)) {
            mStream.setDevice(&mLogFile);
        }
    }
    //---
    if (!mIsRunnedHtml){
        QDir dirHtml(mOutputDirectoryHtml);
        if (!dirHtml.exists()) {
            dirHtml.mkpath(mOutputDirectoryHtml);
        }
        if (mLogFileHtml.isOpen()) {
            mLogFileHtml.close();
        }
        mLogFileHtml.setFileName(mPrevFileNameHtml
                             );
        if (mLogFileHtml.open(QFile::Append)) {
            mStreamHtml.setDevice(&mLogFileHtml);
        }
    }
}

void Logger::stopLog()
{
    mStream.flush();
    mLogFile.close();
    mIsRunned = false;

    mStreamHtml.flush();
    mLogFileHtml.close();
    mIsRunnedHtml = false;
}

void Logger::appendText(const QString& text)
{
    mStream << text;
    mStream << '\n';
}

void Logger::appendTextHtml(const QString &text)
{
    mStreamHtml << text;
    mStreamHtml << '\n';
}

QString Logger::getPrevFileHtml()
{
    return mPrevFileNameHtml;
}
