#ifndef LOGGER_H
#define LOGGER_H

#include <QObject>
#include <QTextStream>
#include <QFile>
#include <QApplication>

///
/// \brief The Logger class текстовый логгер
///
class Logger : public QObject
{
    Q_OBJECT
public:
    explicit Logger(QObject *parent = nullptr);
    ///
    /// \brief setPrefixName установить префикс имени лог файла
    /// \param prefix префикс
    ///
    void setPrefixName(const QString& prefix);
    ///
    /// \brief setOutputDirectory установить выходной каталог для логов
    /// \param dir полное имя директории
    ///
    void setOutputDirectory(const QString& dir);
    ///
    /// \brief startLog начать новый лог
    ///
    void startLog();
    ///
    /// \brief resumeLog продолжить логирование в предыдущий файл
    ///
    void resumeLog();
    ///
    /// \brief stopLog остановить лог
    ///
    void stopLog();
    ///
    /// \brief appendText добавить текстовую строку в текущий начатый лог
    /// \param text текст
    ///
    void appendText(const QString& text);
    void appendTextHtml(const QString& text);
    QString getPrevFileHtml();
private:
    QString mPrefixName;            ///< префикс имени файла
    QString mOutputDirectory;       ///< текущий выходной каталог для логов
    QFile mLogFile;                 ///< текущий начатый файл логов
    QTextStream mStream;            ///< текущий текстовый поток записи
    QString mPrevFileName;          ///< пердыдущее имя файла
    bool mIsRunned = false;         ///< признак работы логгера

    QString mPrefixNameHtml;            ///< префикс имени файла
    QString mOutputDirectoryHtml;       ///< текущий выходной каталог для логов
    QFile mLogFileHtml;                 ///< текущий начатый файл логов
    QTextStream mStreamHtml;            ///< текущий текстовый поток записи
    QString mPrevFileNameHtml;          ///< пердыдущее имя файла
    bool mIsRunnedHtml = false;
};

#endif // LOGGER_H
