#ifndef MEMLOADER_H
#define MEMLOADER_H

#include <QObject>
#include <QVector>

struct MemElement;

///
/// \brief The MemLoader class работа с mem файлом
///
class MemLoader
{
public:
    ///
    /// \brief loadMemFile загрузка мем файла
    /// \param fullname полное имя файла
    /// \param mem содержимое мем-файла
    /// \return результат операции
    ///
    static bool loadMemFile(const QString& fullname, QVector<MemElement>& mem);
    ///
    /// \brief writeToChannel записать содержимое мем-файла в канал в desk
    /// \param channelName имя канала
    /// \param mem содержимое мем-файла
    /// \return результат операции
    ///
    static bool writeToChannel(const QString& channelName, const QVector<MemElement>& mem);
    ///
    /// \brief writeDataToChannel записать содержимое мем-файла в канал в data
    /// \param channelName имя канала
    /// \param mem содержимое мем-файла
    /// \return результат операции
    ///
    static bool writeDataToChannel(const QString& channelName, const QVector<MemElement>& mem);
};

///
/// \brief The MemElement struct элемент мем файла
///
struct MemElement {
    MemElement(){}
    MemElement(int numWord, unsigned int word);
    int mNumWord;           ///< номер слова
    unsigned int mWord;     ///< значение слова
};

#endif // MEMLOADER_H
