#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QMap>
#include <QTimer>
#include "wgt/widgetdevice.h"
#include "interface/iwidgetdevice.h"
#include "entity/onedeviceelectroprogontask.h"
#include "tools/logger.h"

namespace Ui {
class MainWindow;
}

///
/// \brief The MainWindow class главное окно программы
///
class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = nullptr);
    ~MainWindow();
signals:
    /// \brief signalOnCreate событие по старту
    void signalOnCreate();
    ///
    /// \brief signalStartTask команда - старт задач
    ///
    void signalStartTask();
    ///
    /// \brief signalStopTask команда - останов задач
    ///
    void signalStopTask();
    ///
    /// \brief signalPrint команда - печать одной строки
    /// \param line строка
    ///
    void signalPrint(QString line);
    ///
    /// \brief signalPrintLines команда - печать группы строк
    /// \param lines группа строк
    ///
    void signalElectroprogonParametresSet(ElectroprogonParametres params);
    void signalPrintLines(QStringList lines);
public slots:
    ///
    /// \brief onPrint печать одной строки
    /// \param line строка
    ///
    void onPrint(QString line);
    ///
    /// \brief onPrintLines печать группы строк
    /// \param lines группа строк
    ///
    void onPrintLines(QStringList lines);
private slots:
    ///
    /// \brief onCreate слот - событие по старту
    ///
    void onCreate();
    ///
    /// \brief onTimerTracking учёт времени
    ///
    void onTimerTracking();
    ///
    /// \brief onStart старт
    ///
    void onStart();
    ///
    /// \brief onStop стоп
    ///
    void onStop();
    void onShowHtmlLog();
    ///
    /// \brief onFinished подтверждение останова задачи
    ///
    void onFinished();
    ///
    /// \brief onReceiveStatistics приём статистики от задач
    /// \param uid уникальный идентифиикатор платы в программе
    /// \param devStat наработанная статистика
    ///
    void onReceiveStatistics(int uid, DeviceStatistics devStat);
    void onPressKeyBlinked(int key);
private:
    ///
    /// \brief initComboBoxTimeoutElectroprogon инициализация комбобокса с длительностями электропрогона
    ///
    void initComboBoxTimeoutElectroprogon();
    ///
    /// \brief initDeviceList инициализация списка плат и их отображение
    ///
    void initDeviceList();
    ///
    /// \brief timeFromMsToString перевод время в строку
    /// \param time_ms время мс
    /// \return время в строке
    ///
    QString timeFromMsToString(quint64 time_ms);
    ///
    /// \brief registerUserTypes регистрация пользовательских типов
    ///
    void registerUserTypes();
private:
    Ui::MainWindow *ui;
    QMap<int, QString> mDevices;                        ///< девайсы (UID, имя девайса)
    QMap<int, WidgetDevice*> mDeviceWidgets;            ///< виджеты девайсов (UID, виджет девайса)
    QMap<int, OneDeviceElectroprogonTask*> mTasks;      ///< задачи (UID, задача)
    QMap<int, ChannelFinder::DeviceInfo> mDeviceInfoItems;  ///< инфоо о девайсах (UID, инфо)
    QTimer mTimerTracking;                              ///< учёт времени электопрогона
    const quint64 cStepTracking = 1000;                 ///< константа - шаг электропрогона
    quint64 mCurrentTimeTracking;                       ///< текущее время наработки
    quint64 mMaxTimeout;                                ///< время наработки
    int mCountFinished;                                 ///< кол-во подтверждённых завершений
    Logger mLogger;                                     ///< текстовое логирование
    bool mIsAutoMode = false;                           ///< признак авто переключения скоростей
    int mNumA708T;

    // QWidget interface
protected:
    virtual void closeEvent(QCloseEvent *event) override;
};

#endif // MAINWINDOW_H
