#ifndef WIDGETDEVICE_H
#define WIDGETDEVICE_H

#include <QFrame>
#include <QMap>
#include "interface/iwidgetdevice.h"

namespace Ui {
class WidgetDevice;
}

///
/// \brief The WidgetDevice class виджет для отображения наработанной статистики платы
///
class WidgetDevice : public QFrame, public IWidgetDevice
{
    Q_OBJECT

public:
    explicit WidgetDevice(QWidget *parent = nullptr);
    ~WidgetDevice();

    enum class BACKCLR { dark, light };
    ///
    /// \brief setBackClr управление фоном
    /// \param value тёный\светлый
    ///
    void setBackClr(BACKCLR value);
    QString getHtmlText();
    void setKey(int key);

signals:
    void pressedKeyBlinked(int key);

private:
    ///
    /// \brief setVisibleView показать все элементы
    ///
    void setVisibleView();
    ///
    /// \brief setDisableViewUD4x4 убрать лишние элементы для платы 4х4
    ///
    void setDisableViewDEVICE_REV_ID_A708_M_UD2();
    ///
    /// \brief setDisableViewUD6x6 убрать лишние элементы для платы 6х6
    ///
    void setDisableViewDEVICE_REV_ID_A708_UD2();
    ///
    /// \brief setStatistics установка наработанной статистики для платы
    /// \param devStat наработанная статистика
    /// \param devStatParam контрольные параметры статистки
    ///
    void setStatistics(const DeviceStatistics &devStat, const DeviceStatisticsParametres &devStatParam);
    ///
    /// \brief updateStatistics отобразить наработанную статистику для платы
    /// \param devStat наработанная статистика
    ///
    void updateStatistics(const DeviceStatistics &devStat);

private:
    Ui::WidgetDevice *ui;
    int mCurrentRevId = 0;                  ///< текущая ревизия платы
    BACKCLR mCurBackClr = BACKCLR::light;   ///< фон
    int mKey = -1;

    // IWidgetDevice interface
public:
    virtual void setDeviceName(const QString &name) override;
    virtual void setDeviceId(int id) override;
    virtual void setDeviceStatistics(const DeviceStatistics &devStat) override;
    virtual QString getDeviceName() override;
    virtual QStringList getStringStatistics() override;
};

#endif // WIDGETDEVICE_H
